"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserService = void 0;
const common_1 = require("@nestjs/common");
const winston_1 = require("winston");
const mongoose_1 = require("@nestjs/mongoose");
const mongoose_2 = require("mongoose");
const commonServices_1 = require("../services/commonServices");
const api_response_json_1 = require("../../api.response.json");
const jwt = require("jsonwebtoken");
const bcrypt = require("bcrypt");
const axios_1 = require("axios");
const moment = require("moment");
let UserService = class UserService {
    constructor(logger, UserModel, ScriptModel, ScriptVideoModel, AuthTokenModel, PaymentModel, commonServices) {
        this.logger = logger;
        this.UserModel = UserModel;
        this.ScriptModel = ScriptModel;
        this.ScriptVideoModel = ScriptVideoModel;
        this.AuthTokenModel = AuthTokenModel;
        this.PaymentModel = PaymentModel;
        this.commonServices = commonServices;
    }
    async registerUser(userData, res) {
        try {
            let isUser = await this.UserModel.findOne({ email: userData.email });
            if (isUser) {
                return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.EmailUsed);
            }
            let hashedPassword = await bcrypt.hash(userData.password, 10);
            let newUser = await this.UserModel.create({
                first_name: userData.first_name,
                last_name: userData.last_name,
                email: userData.email,
                password: hashedPassword,
                device_token: userData?.device_token,
                device_type: userData?.device_type,
                login_type: userData?.login_type,
            });
            let payload = { userId: newUser._id, isUser: true };
            const token = await jwt.sign(payload, process.env.SECRET_KEY);
            await this.AuthTokenModel.create({ userId: newUser._id, token: token });
            return this.commonServices.returnResponse(res, common_1.HttpStatus.OK, api_response_json_1.message.success.registerSuccess, newUser, token);
        }
        catch (error) {
            console.log('Getting error while register user::', error);
            this.logger.error(`Getting error while register user, Reason :: ${error}`);
            return this.commonServices.returnResponse(res, common_1.HttpStatus.INTERNAL_SERVER_ERROR, api_response_json_1.message.error.serverError);
        }
    }
    async userLogin(credentials, res) {
        try {
            let isUser = await this.UserModel.findOne({ email: credentials.email });
            if (!isUser) {
                return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.notRegistered);
            }
            let isMatched = await bcrypt.compare(credentials.password, isUser.password);
            if (isMatched === false) {
                return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.invalidCredentials);
            }
            let updatedUser = await this.UserModel.findByIdAndUpdate(isUser._id, {
                device_token: credentials?.device_token,
                device_type: credentials?.device_type,
            }, { new: true });
            let payload = { userId: isUser._id, isUser: true };
            const token = await jwt.sign(payload, process.env.SECRET_KEY);
            await this.AuthTokenModel.create({ userId: isUser._id, token: token });
            const today = moment();
            const getPayment = await this.PaymentModel.find({
                userId: isUser?._id,
            });
            const paymentData = getPayment
                .filter((payment) => (payment.status === true ? true : 'true'))
                .map((payment) => ({
                amount: payment.amount,
            }));
            let isPaid = getPayment?.length > 0 ? true : false;
            let isExpire = !getPayment.some((payment) => moment(payment.ExpireDate).isAfter(today));
            return this.commonServices.returnResponse(res, common_1.HttpStatus.OK, api_response_json_1.message.success.loginSuccess, updatedUser, token, isPaid, isExpire, paymentData[0]?.amount ? paymentData[0]?.amount : 0);
        }
        catch (error) {
            console.log('Getting error while user login ::', error);
            this.logger.error(`Getting error while user login, Reason :: ${error}`);
            return this.commonServices.returnResponse(res, common_1.HttpStatus.INTERNAL_SERVER_ERROR, api_response_json_1.message.error.serverError);
        }
    }
    async forgotPassword(userData, res) {
        try {
            let isEmail = await this.UserModel.findOne({ email: userData.email });
            if (!isEmail) {
                return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.notRegistered);
            }
            let otp = await this.commonServices.OTPGenerator(5);
            await this.UserModel.findByIdAndUpdate(isEmail._id, { otp: otp });
            let emailSent = await this.commonServices.sendOtp(isEmail.email, `${isEmail.first_name}  ${isEmail.last_name}`, otp);
            if (emailSent === false) {
                return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.otpNotSent);
            }
            return this.commonServices.returnResponse(res, common_1.HttpStatus.OK, api_response_json_1.message.success.otpSent, otp);
        }
        catch (error) {
            console.log('Getting error while forgot password ::', error);
            this.logger.error(`Getting error while forgot password, Reason :: ${error}`);
            return this.commonServices.returnResponse(res, common_1.HttpStatus.INTERNAL_SERVER_ERROR, api_response_json_1.message.error.serverError);
        }
    }
    async verifyOtp(userData, res) {
        try {
            let isUser = await this.UserModel.findOne({ email: userData.email });
            if (!isUser) {
                return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.notRegistered);
            }
            if (isUser.otp !== userData.otp) {
                return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.invalidOtp);
            }
            await this.UserModel.findByIdAndUpdate(isUser._id, { otp: null }, { new: true });
            return this.commonServices.returnResponse(res, common_1.HttpStatus.OK, api_response_json_1.message.success.otpVerify);
        }
        catch (error) {
            console.log('Getting error while verify otp ::', error);
            this.logger.error(`Getting error while verify otp, Reason :: ${error}`);
            return this.commonServices.returnResponse(res, common_1.HttpStatus.INTERNAL_SERVER_ERROR, api_response_json_1.message.error.serverError);
        }
    }
    async changePassword(userData, res) {
        try {
            const userId = userData._id;
            let email = userData.email;
            let isUser;
            if (userId && !email) {
                isUser = await this.UserModel.findById(userId);
                if (!isUser) {
                    return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.notRegistered);
                }
                let isMatched = await bcrypt.compare(userData.old_password, isUser.password);
                if (isMatched === false) {
                    return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.invalidPass);
                }
            }
            else if (email && !userId) {
                isUser = await this.UserModel.findOne({ email: userData.email });
                if (!isUser) {
                    return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.notRegistered);
                }
            }
            let hashedPassword = await bcrypt.hash(userData.new_password, 10);
            let updateUser = await this.UserModel.findByIdAndUpdate(isUser._id, { password: hashedPassword }, { new: true });
            return this.commonServices.returnResponse(res, common_1.HttpStatus.OK, api_response_json_1.message.success.passwordChange, updateUser);
        }
        catch (error) {
            console.log('Getting error while change password ::', error);
            this.logger.error(`Getting error while change password, Reason :: ${error}`);
            return this.commonServices.returnResponse(res, common_1.HttpStatus.INTERNAL_SERVER_ERROR, api_response_json_1.message.error.serverError);
        }
    }
    async updateProfile(userData, userId, res) {
        try {
            let isUser = await this.UserModel.findById(userId);
            if (isUser.email !== userData.email) {
                let isEmailUsed = await this.UserModel.findOne({
                    email: userData.email,
                });
                if (isEmailUsed) {
                    return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.EmailUsed);
                }
            }
            if (isUser?.profile_image !== userData?.profile_image) {
                let folder = 'public';
                await this.commonServices.deleteFile(isUser.profile_image, folder);
            }
            let updateUser = await this.UserModel.findByIdAndUpdate(userId, {
                email: userData.email,
                profile_image: userData.profile_image || isUser.profile_image,
                first_name: userData.first_name,
                last_name: userData.last_name,
            }, { new: true });
            return this.commonServices.returnResponse(res, common_1.HttpStatus.OK, api_response_json_1.message.success.profileUpdated, updateUser);
        }
        catch (error) {
            console.log('Getting error while update profile ::', error);
            this.logger.error(`Getting error while update profile, Reason :: ${error}`);
            return this.commonServices.returnResponse(res, common_1.HttpStatus.INTERNAL_SERVER_ERROR, api_response_json_1.message.error.serverError);
        }
    }
    async logout(userId, token, res) {
        try {
            let logoutUser = await this.AuthTokenModel.findOneAndDelete({
                userId: new mongoose_2.default.Types.ObjectId(userId),
                token: token,
            });
            if (!logoutUser) {
                return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.logoutError);
            }
            return this.commonServices.returnResponse(res, common_1.HttpStatus.OK, api_response_json_1.message.success.logout);
        }
        catch (error) {
            console.log('Getting error while logout ::', error);
            this.logger.error(`Getting error while logout, Reason :: ${error}`);
            return this.commonServices.returnResponse(res, common_1.HttpStatus.INTERNAL_SERVER_ERROR, api_response_json_1.message.error.serverError);
        }
    }
    async deleteAccount(userId, res) {
        try {
            let isUser = await this.UserModel.findById(userId);
            if (isUser?.profile_image) {
                await this.commonServices.deleteFile(isUser.profile_image, 'public');
            }
            let isScript = await this.ScriptModel.find({
                userId: new mongoose_2.default.Types.ObjectId(userId),
            });
            isScript.map(async (scriptObj) => {
                let ScriptVideos = await this.ScriptVideoModel.find({
                    script_id: new mongoose_2.default.Types.ObjectId(scriptObj._id),
                });
                ScriptVideos.forEach(async (video) => {
                    await this.commonServices.deleteFile(video.thumbnail, 'thumbnail');
                    await this.commonServices.deleteFile(video.video_file, 'videos');
                });
                await this.ScriptVideoModel.deleteMany({
                    script_id: new mongoose_2.default.Types.ObjectId(scriptObj._id),
                });
            });
            let isDeleted = await this.ScriptModel.deleteMany({
                userId: new mongoose_2.default.Types.ObjectId(userId),
            });
            let deleteUser = await this.UserModel.findByIdAndDelete(userId);
            let deleteToken = await this.AuthTokenModel.deleteMany({
                userId: new mongoose_2.default.Types.ObjectId(userId),
            });
            return this.commonServices.returnResponse(res, common_1.HttpStatus.OK, api_response_json_1.message.success.deleteAcc);
        }
        catch (error) {
            console.log('Getting error while delete account ::', error);
            this.logger.error(`Getting error while delete account, Reason :: ${error}`);
            return this.commonServices.returnResponse(res, common_1.HttpStatus.INTERNAL_SERVER_ERROR, api_response_json_1.message.error.serverError);
        }
    }
    async socialLogin(data, res) {
        try {
            const { access_token, login_type, device_token, device_type, email, first_name, last_name, } = data;
            const url = `https://www.googleapis.com/oauth2/v3/tokeninfo?access_token=${access_token}`;
            let userInfo = {
                profile_image: '',
                login_type: login_type,
                device_type: device_type,
                device_token: device_token,
                email,
                first_name,
                last_name,
            };
            if (login_type == 'apple') {
                let isUser = await this.UserModel.findOne({ email: email });
                if (!isUser) {
                    isUser = await this.UserModel.create({
                        profile_image: '',
                        login_type: login_type,
                        device_type: device_type,
                        device_token: device_token,
                        email: email,
                        first_name: first_name,
                        last_name: last_name,
                    });
                }
                let user_id = isUser._id;
                const payload = {
                    userId: user_id,
                    isUser: true,
                };
                const token = await jwt.sign(payload, process.env.SECRET_KEY);
                let addToken = await this.AuthTokenModel.create({
                    userId: new mongoose_2.default.Types.ObjectId(user_id),
                    token: token,
                });
                let userData = await this.UserModel.findByIdAndUpdate(user_id, {
                    device_token: device_token,
                    device_type: device_type,
                    login_type: login_type,
                }, { new: true });
                return this.commonServices.returnResponse(res, common_1.HttpStatus.OK, api_response_json_1.message.success.loginSuccess, userData, addToken.token);
            }
            else {
                let data = await axios_1.default.get(url);
                const user_data = data.data;
                if ((user_data.email_verified = true)) {
                    let email = user_data.email;
                    let is_user = await this.UserModel.findOne({ email: email }, '_id');
                    if (!is_user) {
                        let newUser = await this.UserModel.create({
                            profile_image: '',
                            login_type: login_type,
                            device_type: device_type,
                            device_token: device_token,
                            email: email,
                            first_name: first_name,
                            last_name: last_name,
                        });
                        is_user = await this.UserModel.findById(newUser._id);
                    }
                    let user_id = is_user._id;
                    const payload = {
                        userId: user_id,
                        isUser: true,
                    };
                    const token = await jwt.sign(payload, process.env.SECRET_KEY);
                    await this.AuthTokenModel.create({
                        userId: new mongoose_2.default.Types.ObjectId(user_id),
                        token: token,
                    });
                    let userData = await this.UserModel.findByIdAndUpdate(user_id, {
                        device_token: device_token,
                        device_type: device_type,
                        login_type: login_type,
                    }, { new: true });
                    return this.commonServices.returnResponse(res, common_1.HttpStatus.OK, api_response_json_1.message.success.loginSuccess, userData, token);
                }
                else {
                    return this.commonServices.returnResponse(res, common_1.HttpStatus.BAD_REQUEST, api_response_json_1.message.error.notVerifiedEmail);
                }
            }
        }
        catch (error) {
            console.log('Getting error while social login ::', error.response);
            this.logger.error(`Getting error while social login, Reason :: ${error}`);
            return this.commonServices.returnResponse(res, common_1.HttpStatus.INTERNAL_SERVER_ERROR, api_response_json_1.message.error.serverError);
        }
    }
};
exports.UserService = UserService;
exports.UserService = UserService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, common_1.Inject)('winston')),
    __param(1, (0, mongoose_1.InjectModel)('user')),
    __param(2, (0, mongoose_1.InjectModel)('script')),
    __param(3, (0, mongoose_1.InjectModel)('script-video')),
    __param(4, (0, mongoose_1.InjectModel)('auth-token')),
    __param(5, (0, mongoose_1.InjectModel)('payment')),
    __metadata("design:paramtypes", [winston_1.Logger,
        mongoose_2.Model,
        mongoose_2.Model,
        mongoose_2.Model,
        mongoose_2.Model,
        mongoose_2.Model,
        commonServices_1.commonServices])
], UserService);
//# sourceMappingURL=user.service.js.map